﻿using System.Collections.Generic;
using System.Linq;
using Telerik.Windows.Controls;
using Telerik.Windows.Controls.ScheduleView;

namespace ScheduleView_CustomViewDefinition
{
	public class ConflictCheckingDragDropBehavior : ScheduleViewDragDropBehavior
	{
		private string importantResourceTypeName;

		public string ImportantResourceTypeName
		{
			get
			{
				return this.importantResourceTypeName;
			}
			set
			{
				this.importantResourceTypeName = value;
			}
		}

		public override bool CanDrop(DragDropState state)
		{
			return state.DestinationAppointmentsSource
				.OfType<IAppointment>()
				.Where((IAppointment a) => !state.DraggedAppointments.Contains(a))
				.All((IAppointment a) => state.DestinationSlots.All((Slot s) => !this.AreOverlapping(a, s)));
		}

		public override bool CanResize(DragDropState state)
		{
			return state.DestinationAppointmentsSource
				.OfType<IAppointment>()
				.Where((IAppointment a) => a != state.Appointment)
				.All((IAppointment a) => state.DestinationSlots.All((Slot s) => !this.AreOverlapping(a, s)));
		}

		public override IEnumerable<IOccurrence> CoerceDraggedItems(DragDropState state)
		{
			return state.DraggedAppointments.Where(OwnedByCurrentUser).ToList();
		}

		public override bool CanStartDrag(DragDropState state)
		{
			return base.CanStartDrag(state) && state.DraggedAppointments.All(OwnedByCurrentUser);
		}

		public override bool CanStartResize(DragDropState state)
		{
			return base.CanStartResize(state) && OwnedByCurrentUser(state.Appointment);
		}

		public override void Drop(DragDropState state)
		{
			base.Drop(state);

			foreach (IAppointment a in state.DraggedAppointments)
			{
				this.AddCurrentUser(state, a);
			}
		}

		private static bool OwnedByCurrentUser(IOccurrence item)
		{
			IAppointment appointment = item is IAppointment ? item as IAppointment : (item as Occurrence).Appointment;
			return appointment.Resources.OfType<IResource>().Any((IResource r) => r.ResourceType == "UserName" && r.ResourceName == User.CurrentUser);
		}

		private static bool AreIntersected(IEnumerable<IResource> first, IEnumerable<IResource> other, string importantResourceTypeName)
		{
			IEnumerable<IResource> firstResources = first
				.Where((IResource r) => r.ResourceType == importantResourceTypeName)
				.Distinct();

			IEnumerable<IResource> otherResources = other
				.Where((IResource r) => r.ResourceType == importantResourceTypeName)
				.Distinct();

			return firstResources.Where((IResource r) => otherResources.Contains(r)).Count() == firstResources.Count();
		}

		private bool AreOverlapping(IAppointment appointment, Slot slot)
		{
			return appointment.IntersectsWith(slot) && AreIntersected(appointment.Resources.OfType<IResource>(), slot.Resources.OfType<IResource>(), this.ImportantResourceTypeName);
		}

		private void AddCurrentUser(DragDropState state, IAppointment a)
		{
			this.BeginEdit(state, a);
			a.Resources.Add(new Resource(User.CurrentUser, "UserName"));
			this.TryCommitEdit(state);
		}
	}
}
